"""
This file verifies that the link diagrams (files in this directory ending in
.lnk) we know for class number  principal congruence links are correct - by
comparing it against the triangulations (files ending in .tri) which can be
re-created by
generatePrincipalCongruenceManifolds.py and reginaToSnapPy.py.

Usage:
    python proofCorrectLink.py
or:
    sage -python proofCorrectLink.py

Requires SnapPy version 2.6 or later.

- Matthias Goerner, enischte@gmail.com, 2017-10-20

Update 2022-02-23: Dunfield, Obeidin and Rudd added new link diagrams, see
README.txt. Changed some link diagrams to match those in arXiv:1902.04426v1.
Changed information about required SnapPy version.
"""

from snappy import Manifold

# All principal congruence links we know of
discriminant1 = [
    "pSL_2_plus_0_sqrt_minus_1",
    "pSL_2_plus_1_sqrt_minus_1",
    "pSL_2_plus_2_sqrt_minus_1",
    "pSL_3_plus_1_sqrt_minus_1",
    "pSL_3_plus_0_sqrt_minus_1",
    "pSL_3_plus_2_sqrt_minus_1",
    ]

discriminant2 = [
    "pSL_1_plus_1_sqrt_minus_2",
    "pSL_2_plus_0_sqrt_minus_2",
    "pSL_2_plus_1_sqrt_minus_2",
    "pSL_1_plus_2_sqrt_minus_2",
    ]

discriminant3 = [
    "pSL_2_plus_0_sqrt_minus_3",
    "pSL_2.5_plus_0.5_sqrt_minus_3",
    "pSL_3_plus_0_sqrt_minus_3",
    "pSL_3_plus_1_sqrt_minus_3",
    "pSL_3.5_plus_0.5_sqrt_minus_3",
    ]


discriminant5 = [
    "pSL_3_plus_0_sqrt_minus_5_and_1_plus_1_sqrt_minus_5"    
    ]

discriminant7 = [
    "pSL_0.5_plus_0.5_sqrt_minus_7",
    "pSL_1.5_plus_0.5_sqrt_minus_7",
    "pSL_2_plus_0_sqrt_minus_7",
    "pSL_0_plus_1_sqrt_minus_7",
    "pSL_1_plus_1_sqrt_minus_7",
    "pSL_2.5_plus_0.5_sqrt_minus_7",
    ]

discriminant11 = [
    "pSL_0.5_plus_0.5_sqrt_minus_11",
    "pSL_1.5_plus_0.5_sqrt_minus_11",
    ]

discriminant15 = [
    "pSL_2_plus_0_sqrt_minus_15_and_0.5_plus_0.5_sqrt_minus_15", 
    "pSL_3_plus_0_sqrt_minus_15_and_1.5_plus_0.5_sqrt_minus_15",
    "pSL_4_plus_0_sqrt_minus_15_and_0.5_plus_0.5_sqrt_minus_15",
    "pSL_6_plus_0_sqrt_minus_15_and_-1.5_plus_0.5_sqrt_minus_15",
    "pSL_5_plus_0_sqrt_minus_15_and_2.5_plus_0.5_sqrt_minus_15"
    ]

discriminant19 = [
    "pSL_0.5_plus_0.5_sqrt_minus_19"
]   

discriminant23 = [
    "pSL_2_plus_0_sqrt_minus_23_and_0.5_plus_0.5_sqrt_minus_23",
    "pSL_3_plus_0_sqrt_minus_23_and_0.5_plus_0.5_sqrt_minus_23",
    "pSL_4_plus_0_sqrt_minus_23_and_-1.5_plus_0.5_sqrt_minus_23"
    ]

discriminant31 = [
    "pSL_2_plus_0_sqrt_minus_31_and_0.5_plus_0.5_sqrt_minus_31",
    "pSL_4_plus_0_sqrt_minus_31_and_0.5_plus_0.5_sqrt_minus_31",
    ]

discriminant47 = [
    "pSL_2_plus_0_sqrt_minus_47_and_0.5_plus_0.5_sqrt_minus_47",
    "pSL_3_plus_0_sqrt_minus_47_and_0.5_plus_0.5_sqrt_minus_47",
]

discriminant71 = [
    "pSL_2_plus_0_sqrt_minus_71_and_0.5_plus_0.5_sqrt_minus_71"
]

for f in (discriminant1  +
          discriminant2  +
          discriminant3  +
          discriminant5  +
          discriminant7  +
          discriminant11 +
          discriminant15 +
          discriminant19 +
          discriminant23 +
          discriminant31 +
          discriminant47 +
          discriminant71):

    # Get the triangulation for the principal congruence manifold
    # that we produced with generatePrincipalCongruenceManifold.py
    M = Manifold('../LinkComplementCertificates/' + f + '.tri')

    # Get the link we think is correct and triangulate its complement
    L = Manifold(f + '.lnk')

    # Update 2022-02-23: Added loops to randomize - as it is harder to
    # find geometric triangulations/isometry for the link diagrams given
    # by Dunfield, Obeidin and Rudd.
    for i in range(10):
        if 'positive' in L.solution_type():
            break
        L.randomize()

    for i in range(20):
        if L.is_isometric_to(M):
            print("%s: complement of given link isometric to given triangulation" % f)
            break
        L.randomize()
        M.randomize()
    else:
        raise Exception("%s: SnapPy failed to find isometry between link complement and given triangulation. Try again?" % f)
