// The entire file can be run from the shell with
//     magma LinkComplementHelpers.m LinkComplement2.m
// or from within magma with
//     load "LinkComplementHelpers.m";
//     load "LinkComplement2.m";
// 


// Bianchi group
Bianchi2<a,t,u> := Group<a,t,u|a^2,(t*a)^3,(a*u^-1*a*u)^2,(t,u)>;
// Parabolic elements fixing cusp of the Bianchi orbifold
Bianchi2P := [[ t, u ]];

print "";
print "<1+sqrt(-2)>";

VerifyLink(
    Bianchi2,  // Bianchi group
    Bianchi2P, // For each orbifold cusp, a pair [p1,p2] of parabolics fixing cusp
    [[3,1,1]], // For each orbifold cusp, a triple [n,k,l]. The two parabolics p1^n and p1^k*p2^l are supposed to generate the parabolics of Gamma(I) fixing the respective orbifold cusp
    12,  // Expected size of B(I)
    
    // For each orbifold cusp, a list of "Dehn-fillings" for all the corresponding cusps in the principal congruence manifold
    // Each Dehn-filling is a pair (g, (a, b)). The cusp in the manifold is obtained by applying the group element g (i.e., the manifold cusp is fixed by the peripheral group obtained by conjugating by g). The periperal curve for that cusp corresponds to
    // g * (p1^n)^a * (p1^k*p2^l)^b * g^-1.
    
    [[ <Id(Bianchi2), [ 0, 1]>,
       <a,            [ 0, 1]>,
       <t*a,          [ 0, 1]>,
       <t^-1*a,       [-1, 1]>]]);

print "";
print "<2>";

VerifyLink(
    Bianchi2, Bianchi2P,
    [[2,0,2]], 48,
    [[ <Id(Bianchi2),         [ 1, 0]>,
       <a,                    [ 0, 1]>,
       <t*a,                  [ 1, 0]>,
       <u*a,                  [ 0, 1]>,
       
       <t*u*a,                [ 1, 0]>,
       <a*u*a,                [ 1, 0]>,
       <a*u^-1*a*t^-1*a*u*a,  [ 1, 0]>,
       <t*a*u*a,              [ 0, 1]>,

       <a*u*t^-1*a,           [ 0, 1]>,
       <t^-1*a*u*t^-1*a,      [ 0, 1]>,
       <u^-1*a*u*t^-1*a,      [ 0, 1]>,
       <t^-1*u^-1*a*u*t^-1*a, [ 0, 1]>]]);

print "";
print "<2+sqrt(-2)>";

r:=(t^2*a*t^-2*a); 
s:=(a*t^-2*a*t^2);

VerifyLink(
    Bianchi2, Bianchi2P,
    [[6,2,1]], 72,
    [[ <Id(Bianchi2), [-1, 1]>,
       <r,            [ 0, 1]>,
       <s,            [ 0, 1]>,
       <r*s,          [ 0, 1]>,

       <(t*a),        [ 0, 1]>,
       <r*(t*a),      [ 0, 1]>,
       <s*(t*a),      [ 0, 1]>,
       <r*s*(t*a),    [ 0, 1]>,
          
       <(t*a)^-1,     [ 0, 1]>,
       <r*(t*a)^-1,   [-1, 1]>,
       <s*(t*a)^-1,   [-1, 1]>,
       <r*s*(t*a)^-1, [-1, 1]>]]);

print "";
print "<1+2*sqrt(-2)>";

VerifyLink(
    Bianchi2, Bianchi2P,
    [[9,-4,1]], 324,
    [[ <Id(Bianchi2),        [ 0, 1]>,
       <a,                   [ 1, 2]>,
       <t*a,                 [ 1, 2]>,
       <a*t*a,               [ 1, 2]>,

       <t^2*a,               [ 1, 2]>,
       <a*t^2*a,             [ 1, 2]>,
       <t^-2*a,              [ 1, 2]>,
       <a*t^-2*a,            [ 0, 1]>,

       <t^3*a,               [ 1, 2]>,
       <a*t^3*a,             [ 0, 1]>,
       <t^-3*a,              [ 0, 1]>,
       <a*t^-3*a,            [ 1, 2]>,


       <t^4*a,               [ 1, 2]>,
       <a*t^4*a^-1,          [ 0, 1]>,
       <t^-4*a,              [ 1, 2]>,
       <a*t^-4*a,            [ 1, 2]>,

       <t*a*t^-2*a,          [ 0, 1]>,
       <a*t*a*t^-2*a,        [ 0, 1]>,
       <t^-2*a*t^-2*a,       [ 1, 2]>,
       <a*t^-2*a*t^-2*a,     [ 1, 2]>,

       <t^2*a*t^-2*a,        [ 0, 1]>,
       <a*t^2*a*t^-2*a,      [ 0, 1]>,
       <t^-3*a*t^-2*a,       [ 1, 2]>,
       <a*t^-3*a*t^-2*a,     [ 0, 1]>,


       <t^3*a*t^-2*a,        [ 1, 2]>,
       <a*t^3*a*t^-2*a,      [ 1, 2]>,
       <t^4*a*t^-2*a,        [ 0, 1]>,
       <a*t^4*a*t^-2*a,      [ 1, 2]>,
          
       <t*a*t^4*a*t^-1,      [ 1, 2]>,
       <a*t*a*t^4*a*t^-1,    [ 0, 1]>,
       <t^-1*a*t^-4*a*t,     [ 1, 2]>,
       <a*t^-1*a*t^-4*a*t,   [ 0, 1]>,

       <t*a*t^-4*a*t^-1,     [ 0, 1]>,
       <a*t*a*t^-4*a*t^-1,   [ 1, 2]>,
       <t^2*a*t^-4*a*t^-1,   [ 0, 1]>,
       <a*t^2*a*t^-4*a*t^-1, [ 1, 1]>]]);

print "";
print "<3+sqrt(-2)>";

h:=a*u*a*u*t^-1*a;

VerifyLink(
    Bianchi2, Bianchi2P,
    [[11,3,1]], 660,
    [[ <Id(Bianchi2),   [ 0, 1]>,
       <h,              [ 0, 1]>,
       <h^2,            [ 0, 1]>,
       <h^3,            [ 0, 1]>,
       <h^4,            [ 0, 1]>,

       <a,              [ 0, 1]>,
       <h*a,            [-1, 3]>,
       <h^2*a,          [ 0, 1]>,
       <h^3*a,          [-1, 3]>,
       <h^4*a,          [ 0, 1]>,
       
       <t^-1*a,         [ 0, 1]>,
       <h*t^-1*a,       [ 0, 1]>,
       <h^2*t^-1*a,     [-1, 3]>,
       <h^3*t^-1*a,     [ 0, 1]>,
       <h^4*t^-1*a,     [-1, 3]>,
       
       <t*a,            [ 0, 1]>,
       <h*t*a,          [ 0, 1]>,
       <h^2*t*a,        [ 0, 1]>,
       <h^3*t*a,        [ 0, 1]>,
       <h^4*t*a,        [ 0, 1]>,
       
       <t^-2*a,         [ 0, 1]>,
       <h*t^-2*a,       [ 0, 1]>,
       <h^2*t^-2*a,     [ 0, 1]>,
       <h^3*t^-2*a,     [-1, 3]>,
       <h^4*t^-2*a,     [ 0, 1]>,
       
       <t^2*a,          [1,-3]>,
       <h*t^2*a,        [ 0, 1]>,
       <h^2*t^2*a,      [ 0, 1]>,
       <h^3*t^2*a,      [ 0, 1]>,
       <h^4*t^2*a,      [ 0, 1]>,
       
       <t^-3*a,         [-1, 3]>,
       <h*t^-3*a,       [-1, 3]>,
       <h^2*t^-3*a,     [-1, 3]>,
       <h^3*t^-3*a,     [ 0, 1]>,
       <h^4*t^-3*a,     [-1, 3]>,
       
       <t^3*a,          [-1, 3]>,
       <h*t^3*a,        [-1, 3]>,
       <h^2*t^3*a,      [-1, 3]>,
       <h^3*t^3*a,      [ 0, 1]>,
       <h^4*t^3*a,      [-1, 3]>,

       <t^-4*a,         [ 0, 1]>,
       <h*t^-4*a,       [ 0, 1]>,
       <h^2*t^-4*a,     [ 0, 1]>,
       <h^3*t^-4*a,     [ 0, 1]>,
       <h^4*t^-4*a,     [ 0, 1]>,
       
       <t^4*a,          [ 0, 1]>,
       <h*t^4*a,        [ 0, 1]>,
       <h^2*t^4*a,      [ 0, 1]>,
       <h^3*t^4*a,      [ 0, 1]>,
       <h^4*t^4*a,      [ 0, 1]>,
       
       <t^-5*a,         [ 0, 1]>,
       <h*t^-5*a,       [ 0, 1]>,
       <h^2*t^-5*a,     [ 0, 1]>,
       <h^3*t^-5*a,     [ 0, 1]>,
       <h^4*t^-5*a,     [-1, 2]>,
       
       <t^5*a,          [ 0, 1]>,
       <h*t^5*a,        [ 0, 1]>,
       <h^2*t^5*a,      [ 0, 1]>,
       <h^3*t^5*a,      [ 0, 1]>,
       <h^4*t^5*a,      [ 0, 1]>]]);



