import regina

from trigUtils import *
from trigUtils.barycentricCoarsening import *

import os, sys

usage = """Usage: python barycentricCoarsenReginaTrig.py TRIANGULATION.rga
   or: sage -python barycentricCoarsenReginaTrig.py TRIANGULATION.rga

Given a barycentric subdivision as regina triangulation, coarsen it
(reducing the number of tetrahedra by 4).
"""

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print usage
        sys.exit(1)

    name, ext = os.path.splitext(sys.argv[1])

    with TimingMeasurement("total"):

        with TimingMeasurement("opening regina triangulation"):
            T = regina.open(sys.argv[1])

        with TimingMeasurement("barycentric coarsening"):
            U = barycentricCoarsening(T) 

        outfile = name
        if not '__' in outfile:
            outfile += '_'
        outfile += '_barycentricCoarsening.rga'

        with TimingMeasurement("saving regina triangulation"):
            U.save(outfile)

    print "Saved result as:", outfile
    print "Number of tetrahedra:", U.countTetrahedra()
