from bianchiOrbifold import *
from sage.all import QQ, Integer, prod

import sys

def compute_size_of_psl(ideal):
    one = Integer(1)
    two = Integer(2)

    s = ideal.norm() ** 3 * prod(
        [ one - one / f[0].norm() ** 2 for f in ideal.factor()])

    if two in ideal:
        return s
    else:
        return s / two

usage = """Usage: sage -python sizeOfPrincipalCongruenceManifold.py D a b [c d] [e f] ...

Computes the size of of PSL(2,O/I) = SL(2,O/I)/ { +1/-1} and |PSL(2,O/I)|/|I|
where O is the ring of integers of Q(sqrt(-D)) and I the ideal generated by
a + b * sqrt(-D) and (if supplied) c + d * sqrt(-D), e + f * sqrt(-D), ...
D is assumed to be square-free.

The above two sizes also are the degrees of the prinicipal congruence manifold,
respectively, the manifold H^3/Gamma_1 as covers of the Bianchi orbifold.
"""

if __name__ == '__main__':
    
    if len(sys.argv) < 4 or len(sys.argv) % 2 == 1:
        print usage
        sys.exit(1)

    D = Integer(sys.argv[1])
    imagQuadraticFieldUtil = ImagQuadraticFieldUtil(D)

    gens = [
        imagQuadraticFieldUtil.getAlgebraicInteger(QQ(a), QQ(b))
        for a, b in zip(sys.argv[2::2], sys.argv[3::2]) ]

    ideal = imagQuadraticFieldUtil.CNF.ideal(gens)

    sizeOfPsl = compute_size_of_psl(ideal)

    print "|PSL(2, O_d/I)|       =", sizeOfPsl
    print "|PSL(2, O_d/I)| / |I| =", sizeOfPsl / ideal.norm()
