# Copyright 2017 by Matthias Goerner, enischte@gmail.com.
#
# This code is released under the `GNU General Public License, version 2
# <http://www.gnu.org/licenses/gpl-2.0.txt>`_ or (at your option) any
# later version as published by the Free Software Foundation.

"""
Run tests for the bianchiOrbifold and principalCongruenceManifold module.
"""

from bianchiOrbifold import *

from sage.all import Integer, Rational

from principalCongruenceManifold import * 

from principalCongruenceManifold.testBaselines import __path__ as baselinePaths

import sys
import os

def getBaseline(name):
    return open(os.path.join(baselinePaths[0], name)).read().strip()

def checkIsoSig(D, a, b, expectedIsoSig):
    # Generate orbifold
    reginaContainer = generateBianchiOrbifold(D, 2, 2)
    # Unpack the data
    orb = OrbifoldData(reginaContainer)
    
    imagQuadraticFieldUtil = ImagQuadraticFieldUtil(D)
    ideal = Ideal(imagQuadraticFieldUtil,
                  [imagQuadraticFieldUtil.getAlgebraicInteger(a, b)])

    # Make principal congruence manifold
    pm = PrincipalCongruenceManifoldGenerator(orb, ideal)
    
    # Check against given triangulation isomorphism signature
    isoSig = pm.triangulation.isoSig()
    if not isoSig == expectedIsoSig:
        raise Exception("Unexpected isoSig %s..." % isoSig[:200])

# A quick test
print "Checking the case 2 + 0 * sqrt(-2):   ",
sys.stdout.flush()
checkIsoSig(Integer(2), Integer(2), Integer(0),
            getBaseline('pSL_2_plus_0_sqrt_minus_2.isoSig'))
print "Passed"

# For D=2 and small levels z, we get the same results when we switch up
# matrix multiplication.
# To really test things, use D = 19, because things really fail if we use
# the wrong order to multiply two matrices (the resulting principal congruence
# manifold has unglued faces and ideal vertices with non-toroidal link).
print "Checking the case (1 + sqrt(-19)) / 2:",
sys.stdout.flush()
half = Integer(1) / Integer(2)
checkIsoSig(Integer(19), half, half,
            getBaseline('pSL_0.5_plus_0.5_sqrt_minus_19.isoSig'))
print "Passed"
